@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2023, TileDB, Inc
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #define INCLUDE_ONLY_TILEDB_VERSION
@@ -43,6 +27,11 @@ int TileDBDriverIdentifySimplified(GDALOpenInfo *poOpenInfo)
         return TRUE;
     }
 
+    if (poOpenInfo->IsSingleAllowedDriver("TileDB"))
+    {
+        return TRUE;
+    }
+
     const char *pszConfig =
         CSLFetchNameValue(poOpenInfo->papszOpenOptions, "TILEDB_CONFIG");
 
@@ -91,6 +80,7 @@ void TileDBDriverSetCommonMetadata(GDALDriver *poDriver)
     poDriver->SetMetadataItem(GDAL_DCAP_Z_GEOMETRIES, "YES");
     poDriver->SetMetadataItem(GDAL_DMD_SUBDATASETS, "YES");
 
+    poDriver->SetMetadataItem(GDAL_DCAP_CREATE_LAYER, "YES");
     poDriver->SetMetadataItem(GDAL_DCAP_CREATE_FIELD, "YES");
     poDriver->SetMetadataItem(
         GDAL_DMD_CREATIONFIELDDATATYPES,
@@ -136,12 +126,10 @@ void TileDBDriverSetCommonMetadata(GDALDriver *poDriver)
         "   <Option name='TILEDB_TIMESTAMP' scope='raster' type='int' "
         "description='Create "
         "array at this timestamp, the timestamp should be > 0'/>\n"
-        "   <Option name='BOUNDS' scope='raster' type='string' "
-        "description='Specify "
-        "bounds for sparse array, minx, miny, maxx, maxy'/>\n"
-        "   <Option name='CREATE_GROUP' scope='vector' type='boolean' "
-        "description='Whether to create a group for multiple layer support' "
-        "default='NO'/>"
+        "   <Option name='CREATE_GROUP' type='boolean' "
+        "description='Whether to create a group for multiple vector layer "
+        "support (defaults to NO), or raster with overviews "
+        "(defaults to YES)'/>"
         "</CreationOptionList>\n");
 
     // clang-format off
@@ -275,6 +263,11 @@ void TileDBDriverSetCommonMetadata(GDALDriver *poDriver)
     poDriver->SetMetadataItem(GDAL_DCAP_OPEN, "YES");
     poDriver->SetMetadataItem(GDAL_DCAP_CREATE, "YES");
     poDriver->SetMetadataItem(GDAL_DCAP_CREATECOPY, "YES");
+
+    poDriver->SetMetadataItem(
+        "TILEDB_VERSION",
+        STRINGIFY(TILEDB_VERSION_MAJOR) "." STRINGIFY(
+            TILEDB_VERSION_MINOR) "." STRINGIFY(TILEDB_VERSION_PATCH));
 }
 
 /************************************************************************/
