@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "ogr_oci.h"
@@ -52,7 +36,6 @@ constexpr int anEPSGOracleMapping[] = {
 OGROCIDataSource::OGROCIDataSource()
 
 {
-    pszName = nullptr;
     pszDBName = nullptr;
     papoLayers = nullptr;
     nLayers = 0;
@@ -70,7 +53,6 @@ OGROCIDataSource::~OGROCIDataSource()
 {
     int i;
 
-    CPLFree(pszName);
     CPLFree(pszDBName);
 
     for (i = 0; i < nLayers; i++)
@@ -198,8 +180,6 @@ int OGROCIDataSource::Open(const char *pszNewName, char **papszOpenOptionsIn,
         oValidateStmt.Execute(oValidateCmd.GetString());
     }
 
-    pszName = CPLStrdup(pszNewName);
-
     bDSUpdate = bUpdate;
 
     /* -------------------------------------------------------------------- */
@@ -704,8 +684,8 @@ OGRLayer *OGROCIDataSource::ExecuteSQL(const char *pszSQLCommand,
     /*      Use generic implementation for recognized dialects              */
     /* -------------------------------------------------------------------- */
     if (IsGenericSQLDialect(pszDialect))
-        return OGRDataSource::ExecuteSQL(pszSQLCommand, poSpatialFilter,
-                                         pszDialect);
+        return GDALDataset::ExecuteSQL(pszSQLCommand, poSpatialFilter,
+                                       pszDialect);
 
     /* -------------------------------------------------------------------- */
     /*      Ensure any pending stuff is flushed to the database.            */
